package db.cm.config.impl;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Properties;
import java.util.ResourceBundle;
import db.conn.ConnectorFactory;
import db.cm.ConnectionManager;
import db.cm.config.AbstractConfigurator;
import db.util.Instantiator;

public class ResourceBundleConfigurator extends AbstractConfigurator {
		
	private String mRBName=null;
	private ResourceBundle mRB=null;
	
	private String getResourceBundleName(){
		return mRBName;
	}
		
	private void setResourceBundleName(String pRBName) {
		if(pRBName!=null) mRBName=pRBName;
		else {			
			mRBName=System.getProperty("db.cm.config.impl.rbconfig","dbcm");
		}
	}

	private ResourceBundle getResourceBundle(){
		return mRB;
	}
		
	private void setResourceBundle(ResourceBundle pRB) {
		mRB=pRB;	
	}
	
	private ResourceBundle readResourceBundle(String pRBName){
		ResourceBundle mRBNew=null;
		try{
			mRBNew=ResourceBundle.getBundle(pRBName);
		} catch (Exception eE) {
			eE.printStackTrace();
			mRBNew=null;
		}
		return mRBNew;
	}	

	private ArrayList getValues(String pSource){
		boolean mDone=false;
		int mIndex=-1;
		String mTemp,mTemp2;
		ArrayList mList=new ArrayList();
		while(!mDone){
			if(pSource!=null) {				
				mIndex=pSource.indexOf(",");
				if(mIndex==-1) {
					mTemp=pSource.trim();
					if(mTemp!=null&&!("".equals(mTemp))) mList.add(mTemp);
					mDone=true;
				} else {
					mTemp2=pSource.substring(0,mIndex);
					mTemp=null;
					if(mTemp2!=null) mTemp=mTemp2.trim();
					if(mTemp!=null&&!("".equals(mTemp))) mList.add(mTemp);
					mTemp=pSource.substring(mIndex+1);	pSource=mTemp;
				}
			} else mDone=true;
		}
		return mList;
	}
	
	private Properties processSubProperties(ResourceBundle pRB,String pPrefix){
		String mTemp,mTemp2,mTemp3;
		Properties mNewProp=null;
		Enumeration mEnum;
		if(pRB==null||pPrefix==null||("".equals(pPrefix))) return mNewProp;
		mNewProp=new Properties();
		mEnum=pRB.getKeys();
		//Inefficient Way. It Works though.
		while(mEnum.hasMoreElements()){			
			mTemp2=(String)mEnum.nextElement();
			mTemp=null;
			if(mTemp2!=null) mTemp=mTemp2.trim();
			if(mTemp!=null&&!("".equals(mTemp))&&mTemp.startsWith(pPrefix)) {
				mTemp3=mTemp.substring(pPrefix.length());
				if(mTemp3!=null&&!("".equals(mTemp)))
				mNewProp.put(mTemp3,pRB.getObject(mTemp2));
			}				
		}
		return mNewProp;
	}
	
	private void processResourceBundle(ResourceBundle pRB, ConnectionManager pCM){				
		int mIter;
		String mTemp,mTemp2;
		ArrayList mFactoryList;
		ArrayList mConnectionList;
		String mDefName;
		Properties mFactoryMap;
		Properties mConnectionMap;
		Properties mSubProp;
		ConnectorFactory mCF;
		Enumeration mEnum;		
		if(pRB==null||pCM==null) return;
		mFactoryList=getValues(pRB.getString("factories"));
		mConnectionList=getValues(pRB.getString("connections"));
		mDefName=pRB.getString("defconnection");
		mFactoryMap=new Properties();
		mConnectionMap=new Properties();
		for(mIter=0;mIter<mFactoryList.size();++mIter){			
			mTemp2=pRB.getString("factory."+mFactoryList.get(mIter));
			mTemp=null;
			if(mTemp2!=null) mTemp=mTemp2.trim();
			if(mTemp!=null&&!("".equals(mTemp))) {
				if(Instantiator.getInstance().isSubClass(Instantiator.getInstance().forName(mTemp),ConnectorFactory.class)) {
					mFactoryMap.setProperty((String)mFactoryList.get(mIter),mTemp);
				}
			}
		}
		for(mIter=0;mIter<mConnectionList.size();++mIter){
			mTemp2=pRB.getString("connection."+mConnectionList.get(mIter));
			mTemp=null;
			if(mTemp2!=null) mTemp=mTemp2.trim();
			if(mTemp!=null&&!("".equals(mTemp))) {
				if(mFactoryMap.containsKey(mTemp)) {
				    mConnectionMap.setProperty((String)mConnectionList.get(mIter),mTemp);
				}
			}
		}		
		if(mDefName!=null&&!mConnectionMap.containsKey(mDefName)) mDefName=null;
		mEnum=mConnectionMap.keys();
		while(mEnum.hasMoreElements()){			
			mTemp2=(String)mEnum.nextElement();
			mTemp=null;
			if(mTemp2!=null) mTemp=mTemp2.trim();
			if(mTemp!=null&&!("".equals(mTemp))) {
				mSubProp=processSubProperties(pRB,"connection."+mTemp+".");
				mCF=(ConnectorFactory)Instantiator.getInstance().createObject(Instantiator.getInstance().forName(mFactoryMap.getProperty(mConnectionMap.getProperty(mTemp))),ConnectorFactory.class);
				if(mCF!=null) mCF.initializeValues(mSubProp);
				pCM.addConnectorFactory(mTemp,mCF);
				mCF=null;
			}				
		}		
		
		pCM.setDefaultConnectionName(mDefName);
	}
	
	public ResourceBundleConfigurator(){
		setResourceBundleName(null);
		setResourceBundle(readResourceBundle(getResourceBundleName()));
	}
	
	public ResourceBundleConfigurator(String pRBName){
		setResourceBundleName(pRBName);
		setResourceBundle(readResourceBundle(getResourceBundleName()));		
	}
	
	public ResourceBundleConfigurator(ResourceBundle pRB){
		setResourceBundleName(null);
		setResourceBundle(pRB);
	}
		
	public void configure(ConnectionManager pCM){
		if(pCM==null) return;
		synchronized(pCM) {
			super.configure(pCM);
			processResourceBundle(getResourceBundle(),pCM);
		}
	}	
	
}