package org.bits.ec.dbcon;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * DB Update Query Executor
 */
public class DBQuery {

	/**
	 * Fetches a non String Object based on Primary Key.
	 */
	private Object fetchNonStringKeyEntity(Object pKey, String pSrcColumn,
			String pTargetColumn, String pTable) {
		Object mObj = null;
		Connection mCon = null;
		Statement mStmt = null;
		ResultSet mRs = null;
		String mCmdStr = null;
		DBManager mDBM = new DBManager();

		if (pKey == null)
			return mObj;
		if (pSrcColumn == null || "".equals(pSrcColumn))
			return mObj;
		if (pTargetColumn == null || "".equals(pTargetColumn))
			return mObj;
		if (pTable == null || "".equals(pTable))
			return mObj;

		mCon = mDBM.getConnection();
		if (mCon == null)
			return mObj;

		mCmdStr = "select " + pTargetColumn + " from " + pTable + " where "
				+ pSrcColumn + "=" + pKey.toString();

		try {
			mStmt = mCon.createStatement();
			mRs = mStmt.executeQuery(mCmdStr);
			while (mRs.next()) {
				try {
					mObj = mRs.getObject(1);
				} catch (Exception eE) {
					eE.printStackTrace();
					mObj = null;
				}
			}
		} catch (SQLException eSQLE) {
			eSQLE.printStackTrace();
			mObj = null;
		}

		// Clean Up
		if (mRs != null) {
			try {
				mRs.close();
			} catch (SQLException eSQLE) {
				eSQLE.printStackTrace();
			}
		}
		mRs = null;

		if (mStmt != null) {
			try {
				mStmt.close();
			} catch (SQLException eSQLE) {
				eSQLE.printStackTrace();
			}
		}
		mStmt = null;

		mDBM.closeConnection(mCon);
		mCon = null;

		return mObj;
	}

	/**
	 * Fetches an Object based on Primary Key.
	 */
	public Object fetchKeyEntity(Object pKey, String pSrcColumn,
			String pTargetColumn, String pTable) {
		if (pKey == null)
			return null;
		if (pKey instanceof String || pKey instanceof Character)
			return fetchNonStringKeyEntity("\'"
					+ new DBConvert().doDBConvert(pKey.toString()) + "\'",
					pSrcColumn, pTargetColumn, pTable);
		return fetchNonStringKeyEntity(pKey, pSrcColumn, pTargetColumn, pTable);
	}

	/**
	 * Validates a Non String Entity
	 */
	private int validateNonStringEntity(Object pObj, String pSrcColumn,
			String pTable) {
		int mCount = 0;
		Connection mCon = null;
		Statement mStmt = null;
		ResultSet mRs = null;
		String mCmdStr = null;
		DBManager mDBM = new DBManager();

		if (pObj == null)
			return mCount;
		if (pSrcColumn == null || "".equals(pSrcColumn))
			return mCount;
		if (pTable == null || "".equals(pTable))
			return mCount;

		mCon = mDBM.getConnection();
		if (mCon == null)
			return mCount;

		mCmdStr = "select count(*) from " + pTable + " where " + pSrcColumn
				+ "=" + pObj.toString();

		try {
			mStmt = mCon.createStatement();
			mRs = mStmt.executeQuery(mCmdStr);
			while (mRs.next()) {
				try {
					mCount = (int) mRs.getFloat(1);
				} catch (Exception eE) {
					eE.printStackTrace();
					mCount = 0;
				}
			}
		} catch (SQLException eSQLE) {
			eSQLE.printStackTrace();
			mCount = 0;
		}

		// Clean Up
		if (mRs != null) {
			try {
				mRs.close();
			} catch (SQLException eSQLE) {
				eSQLE.printStackTrace();
			}
		}
		mRs = null;

		if (mStmt != null) {
			try {
				mStmt.close();
			} catch (SQLException eSQLE) {
				eSQLE.printStackTrace();
			}
		}
		mStmt = null;

		mDBM.closeConnection(mCon);
		mCon = null;

		return mCount;
	}

	/**
	 * Validates a Entity
	 */
	public int validateEntity(Object pObj, String pSrcColumn, String pTable) {
		int mFlag = 0;
		if (pObj == null)
			return mFlag;
		if (pObj instanceof String || pObj instanceof Character)
			return validateNonStringEntity("\'"
					+ new DBConvert().doDBConvert(pObj.toString()) + "\'",
					pSrcColumn, pTable);
		return validateNonStringEntity(pObj, pSrcColumn, pTable);
	}

	/**
	 * Execute Update Query.
	 */
	public int executeUpdateQuery(String pCmdStr) {
		int mRows = 0;
		Connection mCon = null;
		Statement mStmt = null;
		DBManager mDBM = new DBManager();

		if (pCmdStr == null || "".equals(pCmdStr))
			return mRows;

		mCon = mDBM.getConnection();
		if (mCon == null)
			return mRows;

		try {
			mStmt = mCon.createStatement();
			mRows = mStmt.executeUpdate(pCmdStr);
		} catch (SQLException eSQLE) {
			eSQLE.printStackTrace();
			mRows = 0;
		}

		// Clean Up
		if (mStmt != null) {
			try {
				mStmt.close();
			} catch (SQLException eSQLE) {
				eSQLE.printStackTrace();
			}
		}
		mStmt = null;

		if (mCon != null) {
			try {
				mCon.close();
			} catch (SQLException eSQLE) {
				eSQLE.printStackTrace();
			}
		}
		mCon = null;

		return mRows;
	}
}