package org.bits.ec.entity;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import org.bits.ec.dbcon.DBConvert;
import org.bits.ec.dbcon.DBManager;
import org.bits.ec.dbcon.DBQuery;
import org.bits.ec.protsys.Auth;

/**
 * User Model.
 */
public class UserModel {

	/**
	 * Allowed Status for Role
	 */
	private int isAllowed(String pName) {
		int mFlag = 0;
		Object mObj = null;
		mObj = new DBQuery().fetchKeyEntity(pName, "username", "allow",
				"user_table");
		if (mObj == null || !(mObj instanceof String))
			return mFlag;
		if ("Y".equals((String) mObj))
			mFlag = 1;
		return mFlag;
	}

	/**
	 * Validates User.
	 */
	public int validateUser(String pName) {
		return isAllowed(pName);
	}

	/**
	 * Validates User Password
	 */
	public int validateUserPlainPassword(String pName, String pPassword) {
		return new Auth().validatePlainPassword(pName, pPassword, "username",
				"userpassword", "user_table");
	}

	/**
	 * Validates User Encrypted Password
	 */
	public int validateUserEncryptedPassword(String pName, String pSessionKey,
			String pEncryptedPassword) {
		return new Auth().validateEncryptedPassword(pName, pSessionKey,
				pEncryptedPassword, "username", "userpassword", "user_table");
	}

	/**
	 * Makes User Encrypted Password
	 */
	public String generateUserEncryptedPassword(String pName,
			String pSessionKey, String pPassword) {
		return new Auth().generateEncryptedPassword(pName, pSessionKey,
				pPassword, "username", "userpassword", "user_table");
	}

	/**
	 * User ID for User Name
	 */
	public int getIDForName(String pName) {
		int mUserID = -1;
		Object mObj = null;
		if (validateUser(pName) <= 0)
			return mUserID;
		mObj = new DBQuery().fetchKeyEntity(pName, "username", "userid",
				"user_table");
		if (mObj == null || !(mObj instanceof Integer))
			return mUserID;
		mUserID = ((Integer) mObj).intValue();
		return mUserID;
	}

	/**
	 * User Name for User ID
	 */
	public String getNameForID(int pUserID) {
		Object mObj = null;
		mObj = new DBQuery().fetchKeyEntity(new Integer(pUserID), "userid",
				"username", "user_table");
		if (mObj == null || !(mObj instanceof String))
			return "";
		return (String) mObj;
	}

	/**
	 * Role Allowed Status for User
	 */
	public int isAllowedForRole(String pName, int pRoleID) {
		int mCount = 0;
		Connection mCon = null;
		Statement mStmt = null;
		ResultSet mRs = null;
		String mCmdStr = null;
		DBManager mDBM = new DBManager();

		if (validateUser(pName) <= 0)
			return mCount;
		if (new RoleModel().validateRole(pRoleID) <= 0)
			return mCount;

		mCon = mDBM.getConnection();
		if (mCon == null)
			return mCount;

		mCmdStr = "select count(*) from user_allowed_table uat, user_table ut where ut.username=\'"
				+ new DBConvert().doDBConvert(pName)
				+ "\' and ut.userid=uat.userid and uat.roleid=" + pRoleID;

		try {
			mStmt = mCon.createStatement();
			mRs = mStmt.executeQuery(mCmdStr);
			while (mRs.next()) {
				try {
					mCount = (int) mRs.getFloat(1);
				} catch (Exception eE) {
					eE.printStackTrace();
					mCount = 0;
				}
			}
		} catch (SQLException eSQLE) {
			eSQLE.printStackTrace();
			mCount = 0;
		}

		// Clean Up
		if (mRs != null) {
			try {
				mRs.close();
			} catch (SQLException eSQLE) {
				eSQLE.printStackTrace();
			}
		}
		mRs = null;

		if (mStmt != null) {
			try {
				mStmt.close();
			} catch (SQLException eSQLE) {
				eSQLE.printStackTrace();
			}
		}
		mStmt = null;

		mDBM.closeConnection(mCon);
		mCon = null;

		return mCount;
	}

}