package org.bits.ec.entity;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import org.bits.ec.dbcon.DBConvert;
import org.bits.ec.dbcon.DBManager;
import org.bits.ec.dbcon.DBQuery;

/**
 * Vote Model.
 */
public class VoteModel {

	/**
	 * Has User voted for a Role?
	 */
	public int hasVotedForRoleID(String pName, int pRoleID) {
		int mCount = 0;
		int mUserID = -1;
		Connection mCon = null;
		Statement mStmt = null;
		ResultSet mRs = null;
		String mCmdStr = null;
		DBManager mDBM = new DBManager();
		UserModel mUM = new UserModel();

		if (mUM.isAllowedForRole(pName, pRoleID) <= 0)
			return mCount;

		mUserID = mUM.getIDForName(pName);

		mCon = mDBM.getConnection();
		if (mCon == null)
			return mCount;

		mCmdStr = "select count(*) from role_table r, vote_table v, candidate_table c where v.userid="
				+ mUserID
				+ " and v.candidateid=c.candidateid and c.roleid=r.roleid and r.roleid="
				+ pRoleID;

		try {
			mStmt = mCon.createStatement();
			mRs = mStmt.executeQuery(mCmdStr);
			while (mRs.next()) {
				try {
					mCount = (int) mRs.getFloat(1);
				} catch (Exception eE) {
					eE.printStackTrace();
					mCount = 0;
				}
			}
		} catch (SQLException eSQLE) {
			eSQLE.printStackTrace();
			mCount = 0;
		}

		// Clean Up
		if (mRs != null) {
			try {
				mRs.close();
			} catch (SQLException eSQLE) {
				eSQLE.printStackTrace();
			}
		}
		mRs = null;

		if (mStmt != null) {
			try {
				mStmt.close();
			} catch (SQLException eSQLE) {
				eSQLE.printStackTrace();
			}
		}
		mStmt = null;

		mDBM.closeConnection(mCon);
		mCon = null;

		return mCount;
	}

	/**
	 * Cast a Vote.
	 */
	public int castVote(String pName, int pCandidateID, String pAddress) {
		int mRows = 0;
		int mUserID = -1;
		int mRoleID = -1;
		UserModel mUM = new UserModel();
		CandidateModel mCM = new CandidateModel();
		IPModel mIM = new IPModel();

		if (mIM.validateIP(pAddress) <= 0)
			return mRows;
		if (mUM.validateUser(pName) <= 0)
			return mRows;
		if (mCM.validateCandidate(pCandidateID) <= 0)
			return mRows;

		mUserID = mUM.getIDForName(pName);
		mRoleID = mCM.getRoleIDForID(pCandidateID);

		if (hasVotedForRoleID(pName, mRoleID) <= 0) // This should actually
			// require a lock on the
			// table.
			return new DBQuery()
					.executeUpdateQuery("insert into vote_table values("
							+ pCandidateID + "," + mUserID + ",now(),\'"
							+ new DBConvert().doDBConvert(pAddress) + "\')");
		return mRows;
	}

	/**
	 * Vote Count for Candidate.
	 */
	public synchronized int votesFor(int pCandidateID) {
		int mCount = 0;
		int mRoleID = -1;
		int mUserID = -1;
		String mAddress = null;
		Connection mCon = null;
		Statement mStmt = null;
		ResultSet mRs = null;
		String mCmdStr = null;
		DBManager mDBM = new DBManager();
		CandidateModel mCM = new CandidateModel();
		IPModel mIM = new IPModel();
		UserModel mUM = new UserModel();

		if (mCM.validateCandidate(pCandidateID) <= 0)
			return mCount;

		mRoleID = mCM.getRoleIDForID(pCandidateID);

		mCon = mDBM.getConnection();
		if (mCon == null)
			return mCount;

		mCmdStr = "select userid, ipaddress from vote_table where candidateid="
				+ pCandidateID;

		try {
			mStmt = mCon.createStatement();
			mRs = mStmt.executeQuery(mCmdStr);
			while (mRs.next()) {
				try {
					mUserID = (int) mRs.getFloat(1);
					mAddress = mRs.getString(2);
					if (mUM
							.isAllowedForRole(mUM.getNameForID(mUserID),
									mRoleID) <= 0
							|| mIM.validateIP(mAddress) <= 0)
						;
					else
						++mCount;
				} catch (Exception eE) {
					eE.printStackTrace();
				}
			}
		} catch (SQLException eSQLE) {
			eSQLE.printStackTrace();
			mCount = 0;
		}

		// Clean Up
		if (mRs != null) {
			try {
				mRs.close();
			} catch (SQLException eSQLE) {
				eSQLE.printStackTrace();
			}
		}
		mRs = null;

		if (mStmt != null) {
			try {
				mStmt.close();
			} catch (SQLException eSQLE) {
				eSQLE.printStackTrace();
			}
		}
		mStmt = null;

		mDBM.closeConnection(mCon);
		mCon = null;

		return mCount;
	}

}