package org.bits.ec.util.schemacreate;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.sql.Connection;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.bits.ec.dbcon.DBManager;
import db.query.QueryOutput;

/**
 * Schema Creation.
 */
public class SchemaCreator {

	/**
	 * Default SQL File Name.
	 */
	private static final String DEFSQLINPUTFILE = "execute.sql";

	/**
	 * Default SQL File System Property.
	 */
	private static final String SQLINPUTFILE = "ec.dclsqlfile";

	/**
	 * Default List File Name.
	 */
	private static final String DEFLISTINPUTFILE = "list.xls";

	/**
	 * Default List File System Property.
	 */
	private static final String LISTINPUTFILE = "ec.studentslistfile";

	/**
	 * List Sheet Used.
	 */
	private static final int DEFAULTLISTSHEET = 0;

	/**
	 * Execute SQL Statements in a SQL File.
	 */
	public int executeSQLFile(String pSQLFileName) {
		int mFlag = 0;
		File mIFile = null;

		BufferedReader mIBR = null;
		BufferedWriter mOBW = null;

		String mSQLQuery = null;
		String mTemp = null;

		QueryOutput mQO = new QueryOutput();
		DBManager mDBM = new DBManager();

		Connection mCon = null;

		if (pSQLFileName == null)
			pSQLFileName = System.getProperty(SQLINPUTFILE);

		if (pSQLFileName == null)
			pSQLFileName = DEFSQLINPUTFILE;

		mCon = mDBM.getConnection();

		if (mCon == null) {
			System.err.println("Can't connect to database.");
			return mFlag;
		}

		mIFile = new File(pSQLFileName);
		try {
			mIBR = new BufferedReader(new FileReader(mIFile));
		} catch (FileNotFoundException eFNFE) {
			eFNFE.printStackTrace();
			return mFlag;
		}

		mOBW = new BufferedWriter(new OutputStreamWriter(System.err));

		try {
			while ((mTemp = mIBR.readLine()) != null) {
				mSQLQuery = mTemp.trim();
				if (mSQLQuery != null && !("".equals(mSQLQuery))
						&& !mSQLQuery.startsWith("#")
						&& !mSQLQuery.startsWith("//")) {
					mQO.executeQuery(mCon, mSQLQuery, mOBW);
					mOBW.newLine();
				}
			}
		} catch (IOException eIOE) {
			eIOE.printStackTrace();
		}

		mDBM.closeConnection(mCon);
		mCon = null;

		try {
			mIBR.close();
			mIBR = null;
			mOBW.flush(); // Not close()
			mOBW = null;
		} catch (IOException eIOE) {
			eIOE.printStackTrace();
		}

		mFlag = 1;

		return mFlag;
	}

	/**
	 * Insert User Data from a XLS File.
	 */
	public int executeList(String pListFileName) {
		int mFlag = 0;

		// In Order of Entity Appearance.
		String mIDNo = null;
		// String mName = null;
		// String mSex = null;
		int mRoomNo = 0;
		String mBhawan = null;
		String mPassword = null;

		PasswordGenerator mPG = new PasswordGenerator();
		DataEntry mDE = new DataEntry();

		File mIFile = null;

		HSSFWorkbook mWB = null;
		HSSFSheet mSheet = null;
		HSSFRow mRow = null;
		HSSFCell mCell = null;

		int mFirstRowNum = 0, mHeaderRowNum = 0, mLastRowNum = 0;
		int mRowIter = 0;

		short mFirstCellNum = 0, mHeaderCellNum = 0, mLastCellNum = 0;
		short mCellIter = 0;

		int mCellType = HSSFCell.CELL_TYPE_BLANK;
		Object mObj = null;

		if (pListFileName == null)
			pListFileName = System.getProperty(LISTINPUTFILE);

		if (pListFileName == null)
			pListFileName = DEFLISTINPUTFILE;

		mIFile = new File(pListFileName);

		try {
			mWB = new HSSFWorkbook(new POIFSFileSystem(new FileInputStream(
					mIFile)));
		} catch (FileNotFoundException eFNFE) {
			eFNFE.printStackTrace();
			return mFlag;
		} catch (IOException eIOE) {
			eIOE.printStackTrace();
			return mFlag;
		}

		if (mWB == null)
			return mFlag;
		mSheet = mWB.getSheetAt(DEFAULTLISTSHEET);
		if (mSheet == null)
			return mFlag;

		mFirstRowNum = mSheet.getFirstRowNum();
		mLastRowNum = mSheet.getLastRowNum();
		// Strip Empty Rows.
		for (mRow = null, mRowIter = mFirstRowNum; mRowIter <= mLastRowNum
				&& ((mRow = mSheet.getRow(mRowIter)) == null); ++mRowIter)
			;
		for (mHeaderRowNum = mRowIter; mRowIter <= mLastRowNum
				&& ((mRow = mSheet.getRow(mRowIter)) != null); ++mRowIter) {
			// Skip Header Row.
			if (mRowIter == mHeaderRowNum)
				continue;
			mFirstCellNum = mRow.getFirstCellNum();
			mLastCellNum = mRow.getLastCellNum();
			// Pre-Initialize Entities
			mIDNo = null;
			// mName = null;
			// mSex = null;
			mRoomNo = 0;
			mBhawan = null;
			mPassword = null;
			// Strip Empty Cells.
			for (mCell = null, mCellIter = mFirstCellNum; mCellIter <= mLastCellNum
					&& ((mCell = mRow.getCell(mCellIter)) == null); ++mCellIter)
				;
			for (mHeaderCellNum = mCellIter; mCellIter <= mLastCellNum
					&& ((mCell = mRow.getCell(mCellIter)) != null); ++mCellIter) {
				mObj = null;
				mCellType = mCell.getCellType();
				if (mCellType == HSSFCell.CELL_TYPE_BLANK)
					;
				else if (mCellType == HSSFCell.CELL_TYPE_BOOLEAN)
					mObj = new Boolean(mCell.getBooleanCellValue());
				else if (mCellType == HSSFCell.CELL_TYPE_ERROR)
					;
				else if (mCellType == HSSFCell.CELL_TYPE_FORMULA)
					;
				else if (mCellType == HSSFCell.CELL_TYPE_NUMERIC)
					mObj = new Double(mCell.getNumericCellValue());
				else if (mCellType == HSSFCell.CELL_TYPE_STRING)
					mObj = mCell.getStringCellValue();
				// Process Object
				if (mCellIter == (mHeaderCellNum) && mObj != null
						&& (mObj instanceof String))
					mIDNo = (String) mObj;
				// if (mCellIter == (mHeaderCellNum + 1) && mObj != null &&
				// (mObj instanceof String))
				// mName = (String) mObj;
				// if (mCellIter == (mHeaderCellNum + 2) && mObj != null &&
				// (mObj instanceof String))
				// mSex = (String) mObj;
				if (mCellIter == (mHeaderCellNum + 3) && mObj != null
						&& (mObj instanceof String))
					mBhawan = (String) mObj;
				if (mCellIter == (mHeaderCellNum + 4) && mObj != null
						&& (mObj instanceof Double))
					mRoomNo = (int) ((Double) mObj).doubleValue();
			}
			mPassword = mPG
					.generatePassword(6, 8, "ABCDEFGHIJKLMNOPQRSTUVWXYZ");
			if (mDE.addUserAndRoles(mIDNo, mPassword, mBhawan, mRoomNo) <= 0)
				System.err.println("Error in processing ID No. : " + mIDNo);
			else {
				System.out.println();
				System.out.println("ID No.   : " + mIDNo);
				System.out.println("Room No. : " + mRoomNo + " " + mBhawan);
				System.out.println("Password : " + mPassword);
				System.out.println();
				System.out
						.println("--------------------------------------------------------------------------------");
			}
		}
		mCell = null;
		mRow = null;
		mSheet = null;
		mWB = null;

		mFlag = 1;

		return mFlag;
	}

	/**
	 * Main Program.
	 */
	public static void main(String[] pArgs) {
		String mSQLFileName = null;
		String mListFileName = null;
		SchemaCreator mSC = new SchemaCreator();
		if (pArgs.length > 0) {
			mSQLFileName = pArgs[0];
			if (pArgs.length > 1) {
				mListFileName = pArgs[1];
			}
		}
		if (mSC.executeSQLFile(mSQLFileName) <= 0)
			return;
		if (mSC.executeList(mListFileName) <= 0)
			return;
		System.err.println("Executed Successfully");
	}
}